import {
  Component,
  OnInit,
  ChangeDetectorRef,
  OnDestroy,
  Output,
  EventEmitter
} from '@angular/core';
import { SelectedDept, UserInfoSS } from '@ucap-webmessenger/protocol-query';
import { NGXLogger } from 'ngx-logger';
import { TranslateService } from '@ngx-translate/core';
import { DialogService } from '@ucap-webmessenger/ui';
import { SessionStorageService } from '@ucap-webmessenger/web-storage';
import { Store, select } from '@ngrx/store';

import * as AppStore from '@app/store';
import * as QueryStore from '@app/store/messenger/query';
import * as SyncStore from '@app/store/messenger/sync';
import * as ChatStore from '@app/store/messenger/chat';
import * as StatusStore from '@app/store/messenger/status';
import { Observable, Subscription } from 'rxjs';
import { PresenceType, StatusCode } from '@ucap-webmessenger/core';
import {
  StatusBulkInfo,
  WorkStatusType
} from '@ucap-webmessenger/protocol-status';
import { VersionInfo2Response } from '@ucap-webmessenger/api-public';
import { KEY_VER_INFO } from '@app/types';

@Component({
  selector: 'app-layout-messenger-organization',
  templateUrl: './organization.component.html',
  styleUrls: ['./organization.component.scss']
})
export class OrganizationComponent implements OnInit, OnDestroy {
  @Output()
  openProfile = new EventEmitter<number>();

  sessionVerinfo: VersionInfo2Response;

  isSearch$: Observable<boolean>;
  selectedDepartmentProcessing$: Observable<boolean>;
  selectedDepartment$: Observable<SelectedDept>;

  departmentUserInfoList$: Observable<UserInfoSS[]>;
  searchDepartmentUserInfoList$: Observable<UserInfoSS[]>;

  profileImageRoot: string;
  PresenceType = PresenceType;
  presenceSubscription: Subscription;
  presence: StatusBulkInfo[] = [];
  displayedColumns: string[] = [
    'profile',
    'deptName',
    'companyName',
    'grade',
    'lineNumber',
    'hpNumber',
    'email',
    'responsibilities',
    'workplace'
  ];

  constructor(
    private store: Store<any>,
    private sessionStorageService: SessionStorageService,
    private dialogService: DialogService,
    private translateService: TranslateService,
    private changeDetectorRef: ChangeDetectorRef,
    private logger: NGXLogger
  ) {
    this.sessionVerinfo = this.sessionStorageService.get<VersionInfo2Response>(
      KEY_VER_INFO
    );
  }

  ngOnInit() {
    this.isSearch$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.isSearch)
    );

    this.selectedDepartmentProcessing$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.selectedDepartmentProcessing
      )
    );

    this.selectedDepartment$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.selectedDepartment)
    );

    this.departmentUserInfoList$ = this.store.pipe(
      select(AppStore.MessengerSelector.QuerySelector.departmentUserInfoList)
    );

    this.searchDepartmentUserInfoList$ = this.store.pipe(
      select(
        AppStore.MessengerSelector.QuerySelector.searchDepartmentUserInfoList
      )
    );

    this.presenceSubscription = this.store
      .pipe(
        select(
          AppStore.MessengerSelector.StatusSelector.selectAllStatusBulkInfo
        )
      )
      .subscribe(presence => {
        this.presence = presence;
      });

    this.profileImageRoot = this.sessionVerinfo.profileRoot;
  }

  ngOnDestroy(): void {
    if (!!this.presenceSubscription) {
      this.presenceSubscription.unsubscribe();
    }
  }

  /*******************************
   * ABOUT TABLE
   *******************************/

  getPresence(userInfo: UserInfoSS, type: PresenceType): string {
    const presences = this.presence.filter(p => p.userSeq === userInfo.seq);

    let status: string;
    let rtnClass = '';
    if (!!presences && presences.length > 0) {
      const presence = presences[0];
      switch (type) {
        case PresenceType.PC:
          status = !!presence ? presence.pcStatus : undefined;
          break;
        case PresenceType.MOBILE:
          status = !!presence ? presence.mobileStatus : undefined;
          break;
      }

      if (!!status) {
        switch (status) {
          case StatusCode.OnLine:
            rtnClass = type + 'On';
            break;
          case StatusCode.Away:
            rtnClass = type + 'Out';
            break;
          case StatusCode.Busy:
            rtnClass = type + 'Other';
            break;
          default:
            rtnClass = type + 'Off';
            break;
        }
      }
    } else {
      rtnClass = type + 'Off';
    }

    return rtnClass;
  }
  getPresenceMsg(userInfo: UserInfoSS): string {
    const presences = this.presence.filter(p => p.userSeq === userInfo.seq);

    if (!!presences && presences.length > 0) {
      const presence = presences[0];
      if (
        !!presence &&
        !!presence.statusMessage &&
        presence.statusMessage !== '.'
      ) {
        return presence.statusMessage;
      }
    }
    return '';
  }
  getWorkstatusInfo(userInfo: UserInfoSS, type: string): string {
    let workstatus = userInfo.workstatus;

    const presences = this.presence.filter(p => p.userSeq === userInfo.seq);

    if (!!presences && presences.length > 0) {
      const presence = presences[0];
      if (
        !!presence &&
        !!presence.workstatus &&
        presence.workstatus.trim().length > 0
      ) {
        workstatus = presence.workstatus;
      }
    }

    let text = '';
    // morning-off: 오전 afternoon-off: 오후  day-off: 휴가 long-time: 장기  leave-of-absence: 휴직
    let style = '';
    switch (workstatus) {
      case WorkStatusType.VacationAM:
        style = 'morning-off';
        text = '오전';
        break;
      case WorkStatusType.VacationPM:
        style = 'afternoon-off';
        text = '오후';
        break;
      case WorkStatusType.VacationAll:
        style = 'day-off';
        text = '휴가';
        break;
      case WorkStatusType.LeaveOfAbsence:
        style = 'leave-of-absence';
        text = '휴직';
        break;
      case WorkStatusType.LongtermRefresh:
        style = 'long-time';
        text = '장기';
        break;
    }

    return type === 'text' ? text : style;
  }

  onClickOpenProfile(event: MouseEvent, userSeq: number) {
    event.preventDefault();
    event.stopPropagation();

    this.openProfile.emit(userSeq);
  }
}
